local VERSION = tonumber(("$Revision: 43939 $"):match("%d+"))

local Parrot = Parrot
local Parrot_CombatEvents = Parrot:NewModule("CombatEvents", "Parser-3.0", "AceEvent-2.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-16 16:18:42 -0400 (Mon, 16 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local L = Parrot:L("Parrot_CombatEvents", {
	["Miss!"] = MISS .. "!",
	["Dodge!"] = DODGE .. "!",
	["Parry!"] = PARRY .. "!",
	["Block!"] = BLOCK .. "!",
	["Absorb!"] = ABSORB .. "!",
	["Immune!"] = IMMUNE .. "!",
	["Resist!"] = RESIST .. "!",
	["Reflect!"] = REFLECT .. "!",
	["Interrupt!"] = INTERRUPT .. "!",
	["Pet Miss!"] = PET .. " " .. MISS .. "!",
	["Pet Dodge!"] = PET .. " " .. DODGE .. "!",
	["Pet Parry!"] = PET .. " " .. PARRY .. "!",
	["Pet Block!"] = PET .. " " .. BLOCK .. "!",
	["Pet Absorb!"] = PET .. " " .. ABSORB .. "!",
	["Pet Immune!"] = PET .. " " .. IMMUNE .. "!",
	["Pet Resist!"] = PET .. " " .. RESIST .. "!",
	["Pet Reflect!"] = PET .. " " .. REFLECT .. "!",
	["Pet Interrupt!"] = PET .. " " .. INTERRUPT .. "!",
})

local Gratuity -- AceLibrary("Gratuity-2.0")
local Aura -- AceLibrary("SpecialEvents-Aura-2.0")
local SharedMedia -- AceLibrary("SharedMedia-1.0")

local _G = _G
local UNKNOWN = _G.UNKNOWN
if type(UNKNOWN) ~= "string" then
	UNKNOWN = "Unknown"
end

local _,playerClass = _G.UnitClass("player")

local new, del = Parrot.new, Parrot.del

Parrot_CombatEvents.db = Parrot:AcquireDBNamespace("CombatEvents")
Parrot:RegisterDefaults("CombatEvents", "profile", {
	['*'] = {
		['*'] = {}
	}
})

local fontValidate = {}
local function rebuildFontValidate()
	for k in pairs(fontValidate) do
		fontValidate[k] = nil
	end
	for _,v in ipairs(SharedMedia:List('font')) do
		fontValidate[#fontValidate+1] = v
	end
	table.sort(fontValidate)
	table.insert(fontValidate, 1, L["Inherit"])
end

local Parrot_Display
local onEnableFuncs = {}
local enabled = false
function Parrot_CombatEvents:OnEnable()
	enabled = true
	Parrot_Display = Parrot:GetModule("Display")
	Gratuity = AceLibrary("Gratuity-2.0")
	Aura = AceLibrary("SpecialEvents-Aura-2.0")
	SharedMedia = AceLibrary("SharedMedia-1.0")
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
	
	rebuildFontValidate()
	self:RegisterEvent("SharedMedia_Registered")
end

function Parrot_CombatEvents:SharedMedia_Registered(category, name)
	if category == 'font' then
		rebuildFontValidate()
	end
end

local onDisableFuncs = {}
function Parrot_CombatEvents:OnDisable()
	enabled = false
	for _,v in ipairs(onDisableFuncs) do
		v()
	end
end

local combatEvents = {}

local function refreshEventRegistration(category, name)
	if not enabled then
		return
	end
	local data = combatEvents[category][name]
	local db = Parrot_CombatEvents.db.profile[category][name]
	local disabled = db.disabled
	if disabled == nil then
		disabled = data.defaultDisabled
	end
	local blizzardEvent = data.blizzardEvent
	local parserEvent = data.parserEvent
	if disabled then
		if blizzardEvent then
			if Parrot_CombatEvents:IsEventRegistered(blizzardEvent) then
				Parrot_CombatEvents:UnregisterEvent(blizzardEvent)
			end
		elseif parserEvent then
			Parrot_CombatEvents:UnregisterParserEvent(parserEvent)
		end
	else
		if blizzardEvent then
			Parrot_CombatEvents:RegisterEvent(blizzardEvent, function(...)
				local info = new(...)
				Parrot_CombatEvents:TriggerCombatEvent(category, name, info)
				info = del(info)
			end)
		elseif parserEvent then
			Parrot_CombatEvents:RegisterParserEvent(parserEvent, Parrot_CombatEvents.TriggerCombatEvent, Parrot_CombatEvents, category, name)
		end
	end
end

onEnableFuncs[#onEnableFuncs+1] = function()
	assert(enabled)
	for category, q in pairs(combatEvents) do
		for name, data in pairs(q) do
			refreshEventRegistration(category, name)
		end
	end
end

local function createOption() end

function Parrot_CombatEvents:OnOptionsCreate()
	Parrot.options.args.events = {
		type = 'group',
		name = L["Events"],
		desc = L["Change event settings"],
		args = {
			Incoming = {
				type = 'group',
				name = L["Incoming"],
				desc = L["Incoming events are events which a mob or another player does to you."],
				args = {},
			},
			Outgoing = {
				type = 'group',
				name = L["Outgoing"],
				desc = L["Outgoing events are events which you do to a mob or another player."],
				args = {},
			},
			Notification = {
				type = 'group',
				name = L["Notification"],
				desc = L["Notification events are available to notify you of certain actions."],
				args = {},
			},
		}
	}
	local function getTag(passValue)
		local category, name = (";"):split(passValue, 2)
		return Parrot_CombatEvents.db.profile[category][name].tag or combatEvents[category][name].defaultTag
	end
	local function setTag(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if combatEvents[category][name].defaultTag == value then
			value = nil
		end
		Parrot_CombatEvents.db.profile[category][name].tag = value
	end

	local function hexColorToTuple(color)
		local num = tonumber(color, 16)
		return math.floor(num / 256^2)/255, math.floor((num / 256)%256)/255, (num%256)/255
	end

	local function tupleToHexColor(r, g, b)
		return ("%02x%02x%02x"):format(r * 255, g * 255, b * 255)
	end

	local function getColor(passValue)
		local category, name = (";"):split(passValue, 2)
		return hexColorToTuple(Parrot_CombatEvents.db.profile[category][name].color or combatEvents[category][name].color)
	end
	local function setColor(passValue, r, g, b)
		local category, name = (";"):split(passValue, 2)
		local color = tupleToHexColor(r, g, b)
		local combatEvent = combatEvents[category][name]
		if combatEvent.color == color then
			color = nil
		end
		Parrot_CombatEvents.db.profile[category][name].color = color
	end

	local function getSticky(passValue)
		local category, name = (";"):split(passValue, 2)
		local sticky = Parrot_CombatEvents.db.profile[category][name].sticky
		if sticky ~= nil then
			return sticky
		else
			return combatEvents[category][name].sticky
		end
	end
	local function setSticky(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if (not not combatEvents[category][name].sticky) == value then
			value = nil
		end
		Parrot_CombatEvents.db.profile[category][name].sticky = value
	end

	local function getFontFace(passValue, value)
		local category, name = (";"):split(passValue, 2)
		local font = Parrot_CombatEvents.db.profile[category][name].font
		if font == nil then
			return L["Inherit"]
		else
			return font
		end
	end
	local function setFontFace(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		Parrot_CombatEvents.db.profile[category][name].font = value
	end
	local function getFontSize(passValue, value)
		local category, name = (";"):split(passValue, 2)
		return Parrot_CombatEvents.db.profile[category][name].fontSize
	end
	local function setFontSize(passValue, value)
		local category, name = (";"):split(passValue, 2)
		Parrot_CombatEvents.db.profile[category][name].fontSize = value
	end
	local function getFontSizeInherit(passValue, value)
		local category, name = (";"):split(passValue, 2)
		return Parrot_CombatEvents.db.profile[category][name].fontSize == nil
	end
	local function setFontSizeInherit(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value then
			Parrot_CombatEvents.db.profile[category][name].fontSize = nil
		else
			Parrot_CombatEvents.db.profile[category][name].fontSize = 18
		end
	end
	local function getFontOutline(passValue, value)
		local category, name = (";"):split(passValue, 2)
		local outline = Parrot_CombatEvents.db.profile[category][name].fontOutline
		if outline == nil then
			return L["Inherit"]
		else
			return outline
		end
	end
	local function setFontOutline(passValue, value)
		local category, name = (";"):split(passValue, 2)
		if value == L["Inherit"] then
			value = nil
		end
		Parrot_CombatEvents.db.profile[category][name].fontOutline = value
	end
	local function getEnable(passValue)
		local category, name = (";"):split(passValue, 2)
		local disabled = Parrot_CombatEvents.db.profile[category][name].disabled
		if disabled == nil then
			disabled = combatEvents[category][name].defaultDisabled
		end
		return not disabled
	end
	local function setEnable(passValue, value)
		local category, name = (";"):split(passValue, 2)
		local disabled = not value
		if (not not combatEvents[category][name].defaultDisabled) == disabled then
			disabled = nil
		end
		Parrot_CombatEvents.db.profile[category][name].disabled = disabled

		refreshEventRegistration(category, name)
	end

	local fontOutlineValidate = {
		NONE = L["None"],
		OUTLINE = L["Thin"],
		THICKOUTLINE = L["Thick"],
		[L["Inherit"]] = L["Inherit"],
	}
	
	function createOption(category, name)
		local localName = combatEvents[category][name].localName
		Parrot.options.args.events.args[category].args[name] = {
			type = 'group',
			name = localName,
			desc = localName,
			args = {
				tag = {
					name = L["Tag"],
					desc = L["Tag to show for the current event."],
					type = 'text',
					usage = L["<Tag>"],
					get = getTag,
					set = setTag,
					passValue = category .. ";" .. name,
				},
				color = {
					name = L["Color"],
					desc = L["Color of the text for the current event."],
					type = 'color',
					get = getColor,
					set = setColor,
					passValue = category .. ";" .. name,
				},
				sticky = category == "Notification" and {
					name = L["Sticky"],
					desc = L["Whether the current event should be classified as \"Sticky\""],
					type = 'toggle',
					get = getSticky,
					set = setSticky,
					passValue = category .. ";" .. name,
				} or nil,
				font = {
					type = 'group',
					name = L["Custom font"],
					desc = L["Custom font"],
					args = {
						fontface = {
							type = 'text',
							name = L["Font face"],
							desc = L["Font face"],
							validate = fontValidate,
							get = getFontFace,
							set = setFontFace,
							passValue = category .. ";" .. name,
							order = 1,
						},
						fontSizeInherit = {
							type = 'toggle',
							name = L["Inherit font size"],
							desc = L["Inherit font size"],
							get = getFontSizeInherit,
							set = setFontSizeInherit,
							passValue = category .. ";" .. name,
							order = 2,
						},
						fontSize = {
							type = 'range',
							name = L["Font size"],
							desc = L["Font size"],
							min = 12,
							max = 30,
							step = 1,
							get = getFontSize,
							set = setFontSize,
							disabled = getFontSizeInherit,
							passValue = category .. ";" .. name,
							order = 3,
						},
						fontOutline = {
							type = 'text',
							name = L["Font outline"],
							desc = L["Font outline"],
							get = getFontOutline,
							set = setFontOutline,
							validate = fontOutlineValidate,
							passValue = category .. ";" .. name,
							order = 4,
						},
					}
				},
				enable = {
					order = -1,
					type = 'toggle',
					name = L["Enabled"],
					desc = L["Enable the current event."],
					get = getEnable,
					set = setEnable,
					passValue = category .. ";" .. name,
				}
			}
		}
	end
	
	for category, q in pairs(combatEvents) do
		for name, data in pairs(q) do
			createOption(category, name)
		end
	end
end

function Parrot_CombatEvents:RegisterCombatEvent(data)
	self = Parrot_CombatEvents -- so people can do Parrot:RegisterCombatEvent
	AceLibrary.argCheck(self, data, 2, "table")
	local category = data.category
	if type(category) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. category must be a %q, got %q."):format("string", type(category)), 2)
	end
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local defaultTag = data.defaultTag
	if type(defaultTag) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. defaultTag must be a %q, got %q."):format("string", type(defaultTag)), 2)
	end
	local parserEvent = data.parserEvent
	if parserEvent and type(parserEvent) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. parserEvent must be a %q or nil, got %q."):format("table", type(parserEvent)), 2)
	end
	local blizzardEvent = data.blizzardEvent
	if blizzardEvent and type(blizzardEvent) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent must be a %q or nil, got %q."):format("string", type(blizzardEvent)), 2)
	end
	if parserEvent and blizzardEvent then
		error("Bad argument #2 to `RegisterCombatEvent'. blizzardEvent and parserEvent cannot coexist.", 2)
	end
	local tagTranslations = data.tagTranslations
	if tagTranslations and type(tagTranslations) ~= "table" then
		error(("Bad argument #2 to `RegisterCombatEvent'. tagTranslations must be a %q or nil, got %q."):format("table", type(tagTranslations)), 2)
	end
	local color = data.color
	if type(color) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. color must be a %q, got %q."):format("string", type(color)), 2)
	end
	if not combatEvents[category] then
		combatEvents[category] = new()
	end
	combatEvents[category][name] = data
	refreshEventRegistration(category, name)
	
	createOption(category, name)
end
Parrot.RegisterCombatEvent = Parrot_CombatEvents.RegisterCombatEvent

local handler__translation
local handler__info
local function handler(literal)
	local inner = literal:sub(2, -2)
	local value = handler__translation[inner]
	if value then
		if type(value) == "function" then
			return tostring(value(handler__info) or UNKNOWN)
		else
			return tostring(handler__info[value] or UNKNOWN)
		end
	else
		return "[" .. inner:gsub("(%b[])", handler) .. "]"
	end
end

local modifierColors = {
	absorb = "ffff00",
	block = "7f00ff",
	resist = "7f007f",
	vulnerable = "7f7fff",
	overheal = "00af7f",
	glancing = "ff0000",
	crushing = "7f0000",
}

local absorbTranslation = { Amount = function(info)
	return "|cff" .. modifierColors.absorb .. info.absorbAmount .. "|r"
end }
local blockTranslation = { Amount = function(info)
	return "|cff" .. modifierColors.block .. info.blockAmount .. "|r"
end }
local resistTranslation = { Amount = function(info)
	return "|cff" .. modifierColors.resist .. info.resistAmount .. "|r"
end }
local vulnerableTranslation = { Amount = function(info)
	return "|cff" .. modifierColors.vulnerable .. info.vulnerableAmount .. "|r"
end }
local overhealTranslation = { Amount = function(info)
	return "|cff" .. modifierColors.overheal .. info.overhealAmount .. "|r"
end }
local glancingTranslation = { Text = function(info)
	return "|r" .. info[1] .. "|cff" .. modifierColors.glancing
end }
local crushingTranslation = { Text = function(info)
	return "|r" .. info[1] .. "|cff" .. modifierColors.crushing
end }

function Parrot_CombatEvents:TriggerCombatEvent(category, name, info)
	self = Parrot_CombatEvents -- so people can do Parrot:TriggerCombatEvent
	if not Parrot:IsModuleActive(self) then
		return
	end
	if type(category) ~= "string" then
		error(("Bad argument #2 to `TriggerCombatEvent'. %q expected, got %q."):format("string", type(category)), 2)
		return
	end
	local data = combatEvents[category]
	if not data then
		error(("Bad argument #2 to `TriggerCombatEvent'. %q is an unknown category."):format(category), 2)
		return
	end
	if type(name) ~= "string" then
		error(("Bad argument #3 to `TriggerCombatEvent'. %q expected, got %q."):format("string", type(name)), 2)
		return
	end
	data = data[name]
	if not data then
		error(("Bad argument #3 to `TriggerCombatEvent'. %q is an unknown name for category %q."):format(name, category), 2)
		return
	end
	
	local db = self.db.profile[category][name]
	local disabled = db.disabled
	if disabled == nil then
		disabled = data.defaultDisabled
	end
	if disabled then
		return
	end
	
	if type(info) ~= "table" then
		error(("Bad argument #4 to `TriggerCombatEvent'. %q expected, got %q."):format("table", type(info)), 2)
		return
	end
	
	local sticky = false
	if data.canCrit then
		sticky = info.isCrit
	elseif category == "Notification" then
		sticky = db.sticky
		if sticky == nil then
			sticky = data.sticky
		end
	end
	local text = db.tag or data.defaultTag
	handler__translation = data.tagTranslations
	handler__info = info
	if handler__translation then
		text = text:gsub("(%b[])", handler)
	end
	local t = new(text)
	local overhealAmount = info.overhealAmount
	if overhealAmount and overhealAmount >= 1 then
		handler__translation = overhealTranslation
		t[#t+1] = L[" ([Amount] overheal)"]:gsub("(%b[])", handler)
		text = table.concat(t)
	else
		local absorbAmount = info.absorbAmount
		if absorbAmount and absorbAmount >= 1 then
			handler__translation = absorbTranslation
			t[#t+1] = L[" ([Amount] absorbed)"]:gsub("(%b[])", handler)
		end
		local blockAmount = info.blockAmount
		if blockAmount and blockAmount >= 1 then
			handler__translation = blockTranslation
			t[#t+1] = L[" ([Amount] blocked)"]:gsub("(%b[])", handler)
		end
		local resistAmount = info.resistAmount
		if resistAmount and resistAmount >= 1 then
			handler__translation = resistTranslation
			t[#t+1] = L[" ([Amount] resisted)"]:gsub("(%b[])", handler)
		end
		local vulnerableAmount = info.vulnerableAmount
		if vulnerableAmount and vulnerableAmount >= 1 then
			handler__translation = vulnerableTranslation
			t[#t+1] = L[" ([Amount] vulnerable)"]:gsub("(%b[])", handler)
		end
		text = table.concat(t)
		if info.isGlancing then
			handler__translation = glancingTranslation
			handler__info = new(text)
			text = "|cff" .. modifierColors.glancing .. L["[Text] (glancing)"]:gsub("(%b[])", handler)
			handler__info = del(handler__info)
		elseif info.isCrushing then
			handler__translation = crushingTranslation
			handler__info = new(text)
			text = "|cff" .. modifierColors.crushing .. L["[Text] (crushing)"]:gsub("(%b[])", handler)
			handler__info = del(handler__info)
		end
	end
	t = del(t)
	handler__translation = nil
	handler__info = nil
	Parrot_Display:ShowMessage(text, category, sticky, db.color or data.color, db.font, db.fontSize, db.fontOutline)
end
Parrot.TriggerCombatEvent = Parrot_CombatEvents.TriggerCombatEvent

local damageTypeToColor = {
	["Physical"] = "ffffff",
	["Holy"] = "ffff7f",
	["Fire"] = "ff7f7f",
	["Nature"] = "7fff7f",
	["Frost"] = "7f7fff",
	["Shadow"] = "7f007f",
	["Arcane"] = "ff7fff",
}

local coloredDamageAmount = function(info) return ("|cff%s%s|r"):format(damageTypeToColor[info.damageType], info.amount) end

------------------------------------------------------------------------------
-- Incoming events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ff0000", -- red
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "([Name]) -[Amount]",
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ff0000", -- red
	throttle = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = "([Skill]) " .. L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = "([Skill]) " .. L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = "([Skill]) " .. L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Ability Blocks",
	localName = L["Ability Blocks"],
	defaultTag = "([Skill]) " .. L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = "([Skill]) " .. L["Resist!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "7f007f", -- purple
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = "([Skill]) " .. L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = "([Skill]) " .. L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = "([Skill]) " .. L["Reflect!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "7f007f", -- purple
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = "([Skill]) " .. L["Interrupt!"],
	parserEvent = {
		eventType = "Interrupt",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	color = "00ff00", -- green
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "([Skill] - [Name]) +[Amount]",
	parserEvent = {
		eventType = "Heal",
		recipientID = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "sourceName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
	color = "00ff00", -- green
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Incoming",
	name = "Environmental Damage",
	localName = L["Environmental Damage"],
	defaultTag = "-[Amount] [Type]",
	parserEvent = {
		eventType = "Environmental",
		recipientID = "player",
	},
	tagTranslations = {
		Amount = coloredDamageAmount,
		Type = "hazardTypeLocal",
	},
	throttle = true,
	color = "ff0000", -- red
}

------------------------------------------------------------------------------
-- Outgoing events -----------------------------------------------------------
------------------------------------------------------------------------------
Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Damage",
	localName = L["Melee Damage"],
	defaultTag = "[Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	color = "ffffff", -- white
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Misses",
	localName = L["Melee Misses"],
	defaultTag = L["Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Dodges",
	localName = L["Melee Dodges"],
	defaultTag = L["Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Parries",
	localName = L["Melee Parries"],
	defaultTag = L["Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Blocks",
	localName = L["Melee Blocks"],
	defaultTag = L["Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Absorbs",
	localName = L["Melee Absorbs"],
	defaultTag = L["Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Immunes",
	localName = L["Melee Immunes"],
	defaultTag = L["Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Melee Evades",
	localName = L["Melee Evades"],
	defaultTag = L["Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Damage",
	localName = L["Skill Damage"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill DoTs",
	localName = L["Skill DoTs"],
	defaultTag = "[Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
		isDoT = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	throttle = true,
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Misses",
	localName = L["Ability Misses"],
	defaultTag = L["Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Dodges",
	localName = L["Ability Dodges"],
	defaultTag = L["Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Parries",
	localName = L["Ability Parries"],
	defaultTag = L["Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Ability Blocks",
	localName = L["Ability Blocks"],
	defaultTag = L["Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Resists",
	localName = L["Spell Resists"],
	defaultTag = L["Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Absorbs",
	localName = L["Skill Absorbs"],
	defaultTag = L["Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Immunes",
	localName = L["Skill Immunes"],
	defaultTag = L["Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Reflects",
	localName = L["Skill Reflects"],
	defaultTag = L["Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Spell Interrupts",
	localName = L["Skill Interrupts"],
	defaultTag = L["Interrupt!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Interrupt",
		sourceID = "player",
		recipientID_not = "player",
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
}


Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Skill Evades",
	localName = L["Skill Evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "player",
		recipientID_not = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals",
	localName = L["Heals"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = false,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	color = "00ff00", -- green
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Heals over Time",
	localName = L["Heals over Time"],
	defaultTag = "+[Amount] ([Skill] - [Name])",
	parserEvent = {
		eventType = "Heal",
		sourceID = "player",
		recipientID_not = "player",
		isHoT = true,
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
		Amount = "realAmount",
	},
	throttle = true,
	color = "00ff00", -- green
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Damage",
	localName = L["Pet Melee Damage"],
	defaultTag = "Pet [Amount]",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
	},
	color = "ff7f00", -- orange
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Misses",
	localName = L["Pet Melee Misses"],
	defaultTag = L["Pet Miss!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Dodges",
	localName = L["Pet Melee Dodges"],
	defaultTag = L["Pet Dodge!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Parries",
	localName = L["Pet Melee Parries"],
	defaultTag = L["Pet Parry!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Blocks",
	localName = L["Pet Melee Blocks"],
	defaultTag = L["Pet Block!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Absorbs",
	localName = L["Pet Melee Absorbs"],
	defaultTag = L["Pet Absorb!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Immunes",
	localName = L["Pet Melee Immunes"],
	defaultTag = L["Pet Immune!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Melee Evades",
	localName = L["Pet Melee Evades"],
	defaultTag = L["Pet Evade!"],
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName = false,
	},
	tagTranslations = {
		Name = "recipientName",
	},
	color = "ff7fff", -- pink
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Damage",
	localName = L["Pet Skill Damage"],
	defaultTag = "Pet [Amount] ([Skill])",
	parserEvent = {
		eventType = "Damage",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Amount = coloredDamageAmount,
		Type = "damageTypeLocal",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Misses",
	localName = L["Pet Ability Misses"],
	defaultTag = L["Pet Miss!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Miss",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Dodges",
	localName = L["Pet Ability Dodges"],
	defaultTag = L["Pet Dodge!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Parries",
	localName = L["Pet Ability Parries"],
	defaultTag = L["Pet Parry!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Ability Blocks",
	localName = L["Pet Ability Blocks"],
	defaultTag = L["Pet Block!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "0000ff", -- blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Spell Resists",
	localName = L["Pet Spell Resists"],
	defaultTag = L["Pet Resist!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Resist",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Absorbs",
	localName = L["Pet Skill Absorbs"],
	defaultTag = L["Pet Absorb!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Absorb",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Immunes",
	localName = L["Pet Skill Immunes"],
	defaultTag = L["Pet Immune!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Immune",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Reflects",
	localName = L["Pet Skill Reflects"],
	defaultTag = L["Pet Reflect!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Reflect",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "7f7fff", -- light blue
	canCrit = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Outgoing",
	name = "Pet Skill Evades",
	localName = L["Pet Skill Evades"],
	defaultTag = L["Evade!"] .. " ([Skill])",
	parserEvent = {
		eventType = "Miss",
		missType = "Evade",
		sourceID = "pet",
		recipientID_not = "pet",
		abilityName_not = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = "abilityName",
	},
	color = "ff7fff", -- pink
}

------------------------------------------------------------------------------
-- Notification events -------------------------------------------------------
------------------------------------------------------------------------------

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff Gains",
	localName = L["Debuff Gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffGained",
	tagTranslations = {
		Name = 1,
	},
	color = "007f7f", -- dark cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff Gains",
	localName = L["Buff Gains"],
	defaultTag = "([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffGained",
	tagTranslations = {
		Name = 1,
	},
	color = "b2b200", -- dark yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item Buff Gains",
	localName = L["Item Buff Gains"],
	defaultTag = "([Name])",
	tagTranslations = {
		Name = 1,
	},
	color = "b2b2b2", -- gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Debuff Fades",
	localName = L["Debuff Fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerDebuffLost",
	tagTranslations = {
		Name = 1,
	},
	color = "00d8d8", -- cyan
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Buff Fades",
	localName = L["Buff Fades"],
	defaultTag = "-([Name])",
	blizzardEvent = "SpecialEvents_PlayerBuffLost",
	tagTranslations = {
		Name = 1,
	},
	color = "e5e500", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Item Buff Fades",
	localName = L["Item Buff Fades"],
	defaultTag = "-([Name])",
	tagTranslations = {
		Name = 1,
	},
	color = "e5e5e5", -- light gray
}

local function GetItemBuffName(slot)
	Gratuity:SetInventoryItem("player", slot)
	
	local _, _, buffName = Gratuity:Find("^([^%(]+) %(%d+ [^%)]+%)$", nil, nil, false, true)
	return buffName
end

local mainhandItemBuff
local offhandItemBuff
local function refreshItemBuffs()
	local mainhand, _, _, offhand = GetWeaponEnchantInfo()
	mainhandItemBuff = mainhand and GetItemBuffName(GetInventorySlotInfo("MainHandSlot"))
	offhandItemBuff = offhand and GetItemBuffName(GetInventorySlotInfo("SecondaryHandSlot"))
end
onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:ScheduleRepeatingEvent("Parrot_CombatEvents:ItemBuffCheck", Parrot_CombatEvents.ItemBuffCheck, 0.2, Parrot_CombatEvents)
	
	refreshItemBuffs()
end
function Parrot_CombatEvents:ItemBuffCheck()
	local oldMainhandItemBuff = mainhandItemBuff
	local oldOffhandItemBuff = offhandItemBuff
	refreshItemBuffs()
	if oldMainhandItemBuff ~= mainhandItemBuff then
		if oldMainhandItemBuff then
			-- loss
			local info = new(oldMainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Fades", info)
			info = del(info)
		end
		if mainhandItemBuff then
			-- gain
			local info = new(mainhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Gains", info)
			info = del(info)
		end
	end
	if oldOffhandItemBuff ~= offhandItemBuff then
		if oldOffhandItemBuff then
			-- loss
			local info = new(oldOffhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Fades", info)
			info = del(info)
		end
		if offhandItemBuff then
			-- gain
			local info = new(offhandItemBuff)
			self:TriggerCombatEvent("Notification", "Item Buff Gains", info)
			info = del(info)
		end
	end
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Enter combat",
	localName = L["Enter combat"],
	defaultTag = "+Combat",
	blizzardEvent = "PLAYER_REGEN_DISABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Leave combat",
	localName = L["Leave combat"],
	defaultTag = "-Combat",
	blizzardEvent = "PLAYER_REGEN_ENABLED",
	color = "ffffff", -- white
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power Gain",
	localName = L["Power Gain"],
	defaultTag = L["+[Amount] [Type]"],
	parserEvent = {
		eventType = "Gain",
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = function(info)
			return info.amountGained or info.amount
		end,
		Type = function(info)
			return info.attributeGainedLocal or info.attributeLocal
		end,
		Skill = "abilityName",
		Name = function(info)
			return info.sourceGainedName or info.sourceName
		end,
	},
	color = "ffff00", -- yellow
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterParserEvent({
		eventType = "Leech",
		sourceGainedID = "player",
		attributeGained_in = { "Mana", "Focus", "Rage", "Energy" },
	}, function(info)
		Parrot_CombatEvents:TriggerEvent("Notification", "Power Gain", false, info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Power Loss",
	localName = L["Power Loss"],
	defaultTag = L["-[Amount] [Type]"],
	parserEvent = {
		eventType_in = { "Drain", "Leech" },
		attribute_in = { "Mana", "Focus", "Rage", "Energy" },
		recipientID = "player",
	},
	tagTranslations = {
		Amount = "amount",
		Type = "attributeLocal",
		Skill = "abilityName",
		Name = "sourceName",
	},
	color = "ffff00", -- yellow
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo Point Gain",
	localName = L["Combo Point Gain"],
	defaultTag = L["[Num] CP"],
	tagTranslations = {
		Num = 1
	},
	color = "ff7f00", -- orange
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Combo Points Full",
	localName = L["Combo Points Full"],
	defaultTag = L["[Num] CP Finish It!"],
	tagTranslations = {
		Num = 1
	},
	color = "ff7f00", -- orange
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_CombatEvents:RegisterEvent("PLAYER_COMBO_POINTS", function()
		local num = GetComboPoints()
		if num == 0 then
			return
		end
		local info = new(num)
		Parrot_CombatEvents:TriggerCombatEvent("Notification", num == 5 and "Combo Points Full" or "Combo Point Gain", info)
		info = del(info)
	end)
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Honor Gains",
	localName = L["Honor Gains"],
	defaultTag = "+[Amount] " .. HONOR_CONTRIBUTION_POINTS,
	parserEvent = {
		eventType = "Honor",
		amount_gt = 0,
		isHonorable = true,
	},
	tagTranslations = {
		Amount = "amount",
		Name = "sourceName",
		Rank = "sourceRank",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation Gains",
	localName = L["Reputation Gains"],
	defaultTag = "+[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_gt = 0,
	},
	tagTranslations = {
		Amount = "amount",
		Faction = "faction",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Reputation Losses",
	localName = L["Reputation Losses"],
	defaultTag = "-[Amount] " .. REPUTATION .. " ([Faction])",
	parserEvent = {
		eventType = "Reputation",
		amount_lt = 0,
	},
	tagTranslations = {
		Amount = function(info) return -info.amount end,
		Faction = "faction",
	},
	color = "7f7fb2", -- blue-gray
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Skill Gains",
	localName = L["Skill Gains"],
	defaultTag = "[Skill]: [Amount]",
	parserEvent = {
		eventType = "Skill",
	},
	tagTranslations = {
		Skill = "abilityName",
		Amount = "amount",
	},
	color = "5555ff", -- semi-light blue
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Experience Gains",
	localName = L["Experience Gains"],
	defaultTag = "[Amount] " .. XP,
	parserEvent = {
		eventType = "Experience",
		recipientID = "player",
	},
	tagTranslations = {
		Name = "sourceName",
		Amount = "amount",
	},
	color = "bf4ccc", -- magenta
	sticky = true,
	defaultDisabled = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Player Killing Blows",
	localName = L["Player Killing Blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = true,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "NPC Killing Blows",
	localName = L["NPC Killing Blows"],
	defaultTag = L["Killing Blow!"] .. " ([Name])",
	parserEvent = {
		eventType = "Death",
		sourceID = "player",
		recipientID_not = "player",
		recipientPvP = false,
	},
	tagTranslations = {
		Name = "recipientName",
		Skill = function(info) return info.abilityName or PLAYERSTAT_MELEE_COMBAT end,
	},
	color = "5555ff", -- semi-light blue
	sticky = true,
}

if playerClass == "WARLOCK" then
	Parrot_CombatEvents:RegisterCombatEvent{
		category = "Notification",
		name = "Soul Shard Gains",
		localName = L["Soul Shard Gains"],
		defaultTag = "+[Name]",
		parserEvent = {
			eventType = "Create",
			itemString_match = "item:6265:[%-%d:]+"
		},
		tagTranslations = {
			Name = "itemName",
		},
		color = "990099", -- purple
		sticky = true,
	}
end

Parrot_CombatEvents:RegisterCombatEvent{
	category = "Notification",
	name = "Extra Attacks",
	localName = L["Extra Attacks"],
	defaultTag = L["%s!"]:format("[Skill]"),
	parserEvent = {
		eventType = "Extra Attack",
		recipientID = "player",
		abilityName_not = false,
	},
	tagTranslations = {
		Skill = "abilityName",
	},
	color = "ffff00", -- yellow
	sticky = true,
}
