local VERSION = tonumber(("$Revision: 43937 $"):match("%d+"))

local Parrot = Parrot
local Parrot_TriggerConditions = Parrot:NewModule("TriggerConditions", "AceEvent-2.0", "Parser-3.0")
if Parrot.revision < VERSION then
	Parrot.version = "r" .. VERSION
	Parrot.revision = VERSION
	Parrot.date = ("$Date: 2007-07-16 16:13:22 -0400 (Mon, 16 Jul 2007) $"):match("%d%d%d%d%-%d%d%-%d%d")
end

local AceEvent = AceLibrary("AceEvent-2.0")

local L = Parrot:L("Parrot_TriggerConditions")

local conditions = {}
local lastStates = {}
local secondaryConditions = {}

local onEnableFuncs = {}
local Parrot_Triggers
function Parrot_TriggerConditions:OnEnable()
	Parrot_Triggers = Parrot:GetModule("Triggers")
	for k, v in pairs(conditions) do
		if v.getCurrent then
			lastStates[k] = v.getCurrent()
		end
	end
	
	for _,v in ipairs(onEnableFuncs) do
		v()
	end
end

local onDisableFuncs = {}
function Parrot_TriggerConditions:OnDisable()
	for _,v in ipairs(onDisableFuncs) do
		v()
	end
end

local function RefreshEvents()
	local self = Parrot_TriggerConditions
	self:UnregisterAllEvents()
	self:UnregisterAllParserEvents()
	if not Parrot:IsModuleActive(self) then
		return
	end
	for k, v in pairs(conditions) do
		if v.events then
			for event in pairs(v.events) do
				self:RegisterEvent(event, "EventHandler")
			end
		elseif v.parserEvent then	
			self:RegisterParserEvent(v.parserEvent, "ParserEventHandler", v)
		end
	end
end

function Parrot_TriggerConditions:EventHandler(arg1)
	local event = AceEvent.currentEvent
	for k, v in pairs(conditions) do
		if v.events then
			local arg = v.events[event]
			if arg == true or (arg1 ~= nil and arg == arg1) then
				if v.getCurrent then
					local state = v.getCurrent()
					local name = v.name
					local oldState = lastStates[name]
					lastStates[name] = state
					if state ~= nil and state ~= oldState then
						self:FireTriggerCondition(name, state)
					end
				else
					self:FireTriggerCondition(v.name, arg1)
				end
			end
		end
	end
end

function Parrot_TriggerConditions:ParserEventHandler(data, info)
	self:FireTriggerCondition(data.name, info[data.parserArg])
end

function Parrot_TriggerConditions:FireTriggerCondition(name, arg)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:FireTriggerCondition
	
	if Parrot_Triggers and Parrot:IsModuleActive(Parrot_Triggers) then
		Parrot_Triggers:OnTriggerCondition(name, arg)
	end
end
Parrot.FireTriggerCondition = Parrot_TriggerConditions.FireTriggerCondition

function Parrot_TriggerConditions:RegisterTriggerCondition(data)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:RegisterTriggerCondition
	AceLibrary.argCheck(self, data, 2, "table")
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	local events = data.events
	local parserEvent = data.parserEvent
	if events then
		if type(events) ~= "table" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("table", type(events)), 2)
		end
		local getCurrent = data.getCurrent
		if type(getCurrent) ~= "function" and type(getCurrent) ~= "nil" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q or nil, got %q."):format("function", type(check)), 2)
		end
	elseif parserEvent then
		if type(parserEvent) ~= "table" then
			error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("table", type(parserEvent)), 2)
		end
	end
	if conditions[name] then
		error(("Trigger condition %q already registered"):format(name), 2)
	end
	conditions[name] = data
	RefreshEvents()
end
Parrot.RegisterTriggerCondition = Parrot_TriggerConditions.RegisterTriggerCondition

function Parrot_TriggerConditions:RegisterSecondaryTriggerCondition(data)
	self = Parrot_TriggerConditions -- in case someone calls Parrot:RegisterSecondaryTriggerCondition
	AceLibrary.argCheck(self, data, 2, "table")
	local name = data.name
	if type(name) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. name must be a %q, got %q."):format("string", type(name)), 2)
	end
	local localName = data.localName
	if type(localName) ~= "string" then
		error(("Bad argument #2 to `RegisterCombatEvent'. localName must be a %q, got %q."):format("string", type(localName)), 2)
	end
	secondaryConditions[name] = data
end
Parrot.RegisterSecondaryTriggerCondition = Parrot_TriggerConditions.RegisterSecondaryTriggerCondition

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Enemy Target Health Percent",
	localName = L["Enemy Target Health Percent"],
	defaultParam = 0.5,
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitCanAttack("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
		 	return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Friendly Target Health Percent",
	localName = L["Friendly Target Health Percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("target") or not UnitIsFriend("player", "target") or UnitIsDeadOrGhost("target") then
			return nil
		else
			return UnitHealth("target")/UnitHealthMax("target")
		end
	end,
	events = {
		UNIT_HEALTH = "target",
		UNIT_MAXHEALTH = "target",
		UNIT_FACTION = "target",
		PLAYER_TARGET_CHANGED = true,
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Self Health Percent",
	localName = L["Self Health Percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") then
			return nil
		else
			return UnitHealth("player")/UnitHealthMax("player")
		end
	end,
	events = {
		UNIT_HEALTH = "player",
		UNIT_MAXHEALTH = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Self Power Percent",
	localName = L["Self Power Percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if UnitIsDeadOrGhost("player") then
			return nil
		else
			return UnitMana("player")/UnitManaMax("player")
		end
	end,
	events = {
		UNIT_MANA = "player",
		UNIT_MAXMANA = "player",
		UNIT_ENERGY = "player",
		UNIT_MAXENERGY = "player",
		UNIT_RAGE = "player",
		UNIT_MAXRAGE = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Pet Health Percent",
	localName = L["Pet Health Percent"],
	param = {
		type = "range",
		min = 0,
		max = 1,
		step = 0.01,
		bigStep = 0.05,
		isPercent = true,
	},
	getCurrent = function()
		if not UnitExists("pet") or UnitIsDeadOrGhost("pet") then
			return nil
		end
		return UnitMana("pet")/UnitManaMax("pet")
	end,
	events = {
		UNIT_HEALTH = "pet",
		UNIT_MAXHEALTH = "pet",
		PLAYER_PET_CHANGED = "pet",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Incoming Block",
	localName = L["Incoming Block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Incoming Crit",
	localName = L["Incoming Crit"],
	parserEvent = {
		eventType = "Damage",
		recipientID = "player",
		isCrit = true,
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Incoming Dodge",
	localName = L["Incoming Dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Incoming Parry",
	localName = L["Incoming Parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		recipientID = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Outgoing Block",
	localName = L["Outgoing Block"],
	parserEvent = {
		eventType = "Miss",
		missType = "Block",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Outgoing Crit",
	localName = L["Outgoing Crit"],
	parserEvent = {
		eventType = "Damage",
		sourceID = "player",
		recipientID_not = "player",
		isCrit = true,
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Outgoing Dodge",
	localName = L["Outgoing Dodge"],
	parserEvent = {
		eventType = "Miss",
		missType = "Dodge",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Outgoing Parry",
	localName = L["Outgoing Parry"],
	parserEvent = {
		eventType = "Miss",
		missType = "Parry",
		sourceID = "player",
		recipientID_not = "player",
	},
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Self Buff Gain",
	localName = L["Self Buff Gain"],
	events = {
		SpecialEvents_PlayerBuffGained = true
	}
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Self Debuff Gain",
	localName = L["Self Debuff Gain"],
	events = {
		SpecialEvents_PlayerDebuffGained = true
	}
}

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Target Buff Gain",
	localName = L["Target Buff Gain"],
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType = "Aura",
		isBuff = true,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot_TriggerConditions:FireTriggerCondition("Target Buff Gain", info.abilityName)
		end
	end)
end

Parrot_TriggerConditions:RegisterTriggerCondition {
	name = "Target Debuff Gain",
	localName = L["Target Debuff Gain"],
}

onEnableFuncs[#onEnableFuncs+1] = function()
	Parrot_TriggerConditions:RegisterParserEvent({
		eventType = "Aura",
		isBuff = false,
	}, function(info)
		if UnitExists("target") and info.recipientName == UnitName("target") then
			Parrot_TriggerConditions:FireTriggerCondition("Target Debuff Gain", info.abilityName)
		end
	end)
end
